/*
    Project: TCGINA
    Version: V1.16
    File   : tcg.h

    Copyright (c) 2005-2007 Author of TCGINA DLL. All rights reserved.
    Public Key Fingerprint: 294B A769 4A0A CC05 DAE6 00DD FF47 8C72 4097 67CE

    Covered by TrueCrypt License 2.2 the full text of which is contained
    in the file License.txt included in TCGINA binary and source code
    distribution archives.

    The copyright notice of this source code file and the public key
    fingerprint must be retained in full.
*/

#ifndef __TCG_H_D47DF546F16B490bA6D8523603A5594D
#define __TCG_H_D47DF546F16B490bA6D8523603A5594D

#include "tcpriv.h"
#include "kfi.h"
#include <winwlx.h>
#include <WtsApi32.h>

//#define SUPPORT_NETWORK
//#define SUPPORT_AUTOMOUNT
//#define SUPPORT_KEYFILES
//#define SUPPORT_TCPASSWORD
//#define SUPPORT_SERVICES
//#define SUPPORT_SHARED_MEMORY

#define TCG_AUTOMOUNT_DEFAULT_TIMEOUT (5*60*1000)

#define TCG_REGKEY_NAME \
  TEXT("SOFTWARE\\TCGINA DLL\\{D47DF546-F16B-490b-A6D8-523603A5594D}")
#define TCG_WINLOGON_KEY \
  TEXT("SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Winlogon")

#define TCG_MAX_STRING_LIST 1060

/* session type */
#define TCG_ST_NONE      0
#define TCG_ST_UNKNOWN   1
#define TCG_ST_STANDARD  2
#define TCG_ST_ENCRYPTED 3

/* encrypted profile info status */
#define TCG_EPIS_ALREADY_EXISTS ((DWORD)-2)
#define TCG_EPIS_INVALID        ((DWORD)-1)
#define TCG_EPIS_UNUSED         0
#define TCG_EPIS_LOGGING_OFF    1
#define TCG_EPIS_AUTHENTICATED  2
#define TCG_EPIS_LOGGED_ON      3

#define TCG_MSO_PROTECT_ENCRYPTED_SESSION 0x00000001
#define TCG_MSO_RUNNING_SESSIONS_WARNING  0x00000002

typedef struct
{
    INT                       iStatus;

    TCHAR                     szDomain[MAX_PATH];
    TCHAR                     szUserName[MAX_PATH];

    DWORD                     dwMountedDrives;

    BOOL                      fWaitForNtUserDat;
    TCHAR                     szNtUserDat[MAX_PATH];

#ifdef SUPPORT_NETWORK
    TCHAR                     szNDLocalName[MAX_PATH];
    TCHAR                     szRemoteName[MAX_PATH];
#endif
} TCG_ENCRYPTED_PROFILE_INFO, *PTCG_ENCRYPTED_PROFILE_INFO;

typedef struct {
    DWORD dwProfileCount;
    TCG_ENCRYPTED_PROFILE_INFO rgProfileInfo[1];
} TCG_SHARED_MEMORY, *PTCG_SHARED_MEMORY;

/* the following handle is defined to prevent direct shared memory access */
typedef void * TCG_EPI_HANDLE;

#define TCG_OS_WIN_2000  0x50000
#define TCG_OS_WIN_XP    0x50001
#define TCG_OS_WIN_VISTA 0x60000

typedef struct
{
    LPWSTR                      lpWinsta;
    HANDLE                      hWlx;
    PWLX_DISPATCH_VERSION_1_4   pWlxFuncs;
    PVOID                       pMsginaContext;

#ifdef SUPPORT_AUTOMOUNT
    BOOL                        fAutoMountDone;
#endif
    BOOL                        fShuttingDown;

    DWORD                       dwOSVersion;
    BOOL                        fRemoteSession;
    DWORD                       dwMSOptions;
#ifdef SUPPORT_SHARED_MEMORY
    DWORD                       dwMSCount;
    PTCG_SHARED_MEMORY          pShareMemory;
#endif
    PTCG_ENCRYPTED_PROFILE_INFO pProfileBegin;
    PTCG_ENCRYPTED_PROFILE_INFO pProfileEnd;
    TCG_EPI_HANDLE              hEpi;
} TCG_CONTEXT, *PTCG_CONTEXT;

#define TC_DISPLAY_PASSWORD             0x00000001
#define TC_IGNORE_USER_ACCOUNT_PASSWORD 0x00000002
#define TC_AUTO_REPAIR_PROFILE_PATH     0x00000004

typedef struct
{
    BOOL          fTCFileNameAvailable;

    DWORD         dwTCOptions;
    DWORD         dwTCMountOptions;
    DWORD         dwTCPasswordTimeout;
    DWORD         dwTCPasswordTotalTimeout;
    DWORD         dwTCKeyFileTimeout;
    TCHAR         szTCPath[TCG_MAX_STRING_LIST];
    TCHAR         szTCFileName[MAX_PATH];
    TCHAR         szTCDrive[4];
    TCHAR         szTCFirstAutoMountDrive[4];
    TCHAR         szTCKeyFileNames[TCG_MAX_STRING_LIST];
    TCHAR         szTCKeyFileDrives[26+1];
#ifdef SUPPORT_TCPASSWORD
    TCHAR         szTCPassword[TC_MAX_PASSWORD];
#endif

#ifdef SUPPORT_KEYFILES
    TCHAR         szKFFileName[MAX_PATH];
    TCHAR         szKFDrives[26+1];
    TCHAR         szKFPassword[KFI_MAX_PASSWORD];
    LARGE_INTEGER liKFFileOffset;
    DWORD         dwKFFileSize;
    DWORD         dwKFID;
    DWORD         dwKFNoPasswordDialog;
    DWORD         dwKFMediumTimeout;
    DWORD         dwKFPasswordTimeout;
    DWORD         dwKFPasswordTotalTimeout;
#endif

#ifdef SUPPORT_NETWORK
    TCHAR         szNDLocalName[MAX_PATH];
    TCHAR         szNDRemoteName[MAX_PATH];
    TCHAR         szNDUserName[MAX_PATH];
    TCHAR         szNDPassword[MAX_PATH];
#endif

#ifdef SUPPORT_SERVICES
    TCHAR         szTCStartServices[TCG_MAX_STRING_LIST];
    TCHAR         szTCStopServices[TCG_MAX_STRING_LIST];
#endif
} TCG_SETTINGS, *PTCG_SETTINGS;

typedef struct
{
    KFI_DATA            kdDecryptedData;
    CHAR                aszPassword[KFI_MAX_PASSWORD];
    CHAR                aszHiddenVolumePassword[KFI_MAX_PASSWORD];
    union {
        KFI_ENCRYPTION_DATA ked;
        WCHAR               wszTempPassword[KFI_MAX_PASSWORD];
        CHAR                aszTempPassword[KFI_MAX_PASSWORD];
    } tmp;
} TCG_KEY_DATA, *PTCG_KEY_DATA;

#define GetMsginaContext(pCtx) ((PTCG_CONTEXT)pCtx)->pMsginaContext

extern HINSTANCE g_hinstDLL;

#ifdef __cplusplus
extern "C" {
#endif

/* tcg */
BOOL WINAPI TCG_Logon(PTCG_CONTEXT pCtx, LPCWSTR pwszUserName,
  LPCWSTR pwszPassword, LPCWSTR pwszDomain);
VOID WINAPI TCG_Logoff(PTCG_CONTEXT pCtx);
VOID WINAPI TCG_LoggedOff(PTCG_CONTEXT pCtx);

/* tcgdebug */
#ifdef TC_DEBUG
    VOID WINAPI TCG_DumpContext(PTCG_CONTEXT pCtx);
    VOID WINAPI TCG_DumpHeap(VOID);
    VOID WINAPI TCG_DumpDriveLetters(const char *paszText, DWORD dwDrives);
#else
#   define TCG_DumpContext(x)
#   define TCG_DumpHeap()
#   define TCG_DumpDriveLetters(x,y)
#endif

/* tcgmisc */
void WINAPI TCG_ReadSettings(PTCG_SETTINGS pSettings,
  LPCTSTR pszUserName);
DWORD WINAPI TCG_MapNetworkDrive(LPTSTR pszLocalName,
  LPTSTR pszRemoteName, LPCTSTR pszUserName, LPCTSTR pszPassword);
DWORD WINAPI TCG_UnmapNetworkDrive(LPCTSTR pszLocalName);

/* tcgauto */
BOOL WINAPI TCG_Automount(PTCG_CONTEXT pCtx);

/* tcgmount */
BOOL WINAPI TCG_ProfileImagePathIsValid(LPCTSTR pszRoot,
  LPCTSTR pszUserName);
VOID WINAPI TCG_AutoMountPartitions(TC_HANDLE hTC,
  TCHAR chDriveLetter, PTCG_KEY_DATA ptkd, DWORD dwMountOptions,
  LPCTSTR pszKeyFiles,
  BOOL fProtectHiddenVolume, LPCTSTR pszHiddenVolumeKeyFiles,
  PDWORD pdwMountedDrives);
BOOL WINAPI TCG_Mount(TC_HANDLE hTC, LPCTSTR pszFileName,
  LPCTSTR pszDrive, LPCTSTR pszUserName, LPCTSTR pszNtUserDat,
  PTCG_KEY_DATA ptkd, DWORD dwMountOptions,
  LPCTSTR pszKeyFiles,
  BOOL fProtectHiddenVolume, LPCTSTR pszHiddenVolumeKeyFiles);

/* tcgkeyfile */
BOOL WINAPI TCG_ReadKeyFile(PTCG_CONTEXT pCtx, PTCG_KEY_DATA ptkd,
  PTCG_SETTINGS pts);

/* tcgservices */
BOOL WINAPI TCG_StopService(LPCTSTR pszServiceName);
BOOL WINAPI TCG_StartService(LPCTSTR pszServiceName);
BOOL WINAPI TCG_StopServices(LPCTSTR pszServiceNames);
BOOL WINAPI TCG_StartServices(LPCTSTR pszServiceNames);

/* tcgsharedmem */
#ifdef SUPPORT_SHARED_MEMORY
  PTCG_SHARED_MEMORY WINAPI TCG_InitSharedMemory(PDWORD);
  BOOL WINAPI TCG_LockSharedMemory(void);
  BOOL WINAPI TCG_UnlockSharedMemory(void);
#else
#  define TCG_InitSharedMemory(p) ((PTCG_SHARED_MEMORY)NULL)
#  define TCG_LockSharedMemory() FALSE
#  define TCG_UnlockSharedMemory() FALSE
#endif

#ifdef __cplusplus
}
#endif

#endif
